;; Checks whether the concept already exists in Scone and
;; if that is not the case, adds a new type with the specified
;; features and parent.
(defun new-extracted-type (iname
       parent
       &key (features '()))
   ;; Skip duplicates.  Create the new type-node.
   (let ((e (lookup-element iname)))
     ;; Add the features for this new type-node.
     (if e
      (new-is-a iname parent)
      (progn (setq e (new-type iname parent))
              (dolist (f features)
                (set-element-property e f))))
      e))

;; Checks whether the individual already exists in Scone and
;; creates it if that is not the case.
(defun new-extraction-indv (iname
       parent
       &key (features '()))
   ;; Skip duplicates.  Create the new indv-node.
   (let ((e (lookup-element iname)))
      (if e
        (new-is-a iname parent)
     ;; Add the features for this new indv-node.
        (progn (setq e (new-indv iname parent))
               (dolist (f features)
                 (set-element-property e f))))
      e))

;; Creates the two elements being equated first using the
;; helper function new-wordnet-type and then adds an eq link.
(defun new-extracted-eq (one-all two-all)
  (let ((first-el (apply 'new-wordnet-type one-all))
        (second-el (apply 'new-wordnet-type two-all)))
       (handler-case
          (new-eq first-el second-el)
          (error nil))))

;; Creates the two elements being equated first using the
;; helper function new-wordnet-type and then adds an is-a link.
(defun new-extracted-is-a (one-all two-all)
  (let ((first-el (apply 'new-wordnet-type one-all))
        (second-el (apply 'new-wordnet-type two-all)))
       (handler-case
          (new-is-a first-el second-el)
          (error nil))))

;; Given a list of inames and strings for them, determines whether
;; one already exists as a concept. If so, adds the rest as English
;; spellings. Otherwise, creates a new concept and adds the English
;; spellings - the expected use case is that the first element of l
;; and s is the element to be used as a concept if none already
;; exist in Scone.
(defun add-english-names (l s)
  (let ((i 0)
        (e (nth 0 l)))
    (loop while (and (not (lookup-element e)) (< i (length l))) do
      (setq i (+ i 1))
      (setq e (nth i l)))
    (if (= i (length l))
        (progn (setq i 0)
               (new-wordnet-type (nth 0 l) {thing})))
    (let ((strings (append (subseq s 0 i) (subseq s i (length l)))))
         (setq strings (cons (nth i l) strings))
         (apply 'english strings))))

;; Checks whether a given relation already exists in Scone. If not, it is added.
(defun new-extracted-relation (iname &optional (features nil))
  (let ((e (lookup-element iname)))
    (if (not (relation? e))
      (progn (new-relation iname)
             (dolist (f features)
               (set-element-property e f))))))